//  Engine.h - an engine for the drivetrain.
//
//  Copyright (C) 2001 Sam Varner
//
//  This file is part of Vamos Automotive Simulator.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _ENGINE_H_
#define _ENGINE_H_

#include <vamos/geometry/Conversions.h>

namespace Vamos_Body
{
  //* An engine for the drivetrain.  Although it produces a torque,
  // Engine is not derived from Component because the Engine's torque
  // is a scalar, not a vector.
  class Engine
  {
	// Used to calculate torque in torque_map ().
	double m_torque_factor;

	// Used to calculate torque in torque_map ().
	double m_friction_factor;

	// The rotational inertia of the engine.
	double m_inertia;

	// The rotational speed of the engine.
	double m_rotational_speed;

	// The rate of fuel consumption.
	double m_fuel_consumption;

	// The throttle position.
	double m_gas;

	// true if the gas tank is empty, false otherwise.
	bool m_out_of_gas;

	// The minimum throttle position.
	double m_idle;

	// The rotational speed that the engine is set to when starting.
	double m_start_speed;

	// The engine shuts off if the rotational speed goes below this
	// value. 
	double m_stall_speed;

	// The current torque produced by the engine.
	double m_torque;

	// The impulse calculated in the last call to torque ().
	double m_impulse;

	// true if the clutch is fully engaged, false otherwise.
	bool m_engaged;

	// Return the torque for a given throttle setting, GAS, and engine
	// speed ROTATIONAL_SPEED.
	double torque_map (double gas, double rotational_speed);

	// Set the engine speed to SPEED_IN and calculate the resulting
	// impulse.
	void speed (double speed_in);

  public:
	//** Constructor
	Engine (double torque_factor, 
			double friction_factor, 
			double inertia,
			double fuel_consumption);

	// Handle the input parameters.  GAS is the throttle position.
	// TRANSMISSION_SPEED is the rotational speed of the transmission
	// side of the clutch.  DRAG is the torque due to friction when
	// the clutch is not fully engaged.  ENGAGED is true when the
	// clutch is fully engaged, false otherwise.
	void input (double gas, double drag, double transmission_speed, 
				bool engaged);

	// Advance the engine in time by TIME.
	void propagate (double time);
  
	// Return the current rotational speed in radians per second.
	double rotational_speed () const { return m_rotational_speed; }

	// Return the current torque.
	double torque () const { return m_torque; }

	// Return the current rate of fuel consumption. 
	double fuel_rate () const 
	{ return m_fuel_consumption * m_rotational_speed * m_gas; }

	// Return true if the fuel tank is empty, false otherwise.
	void out_of_gas (bool out) { m_out_of_gas = out; }

	// Start the engine.
	void start () { speed (m_start_speed); }
  };
}

#endif // !_ENGINE_H_
