//	Particle.h - a massive particle which is part of a rigid body.
//
//  Copyright (C) 2001--2002 Sam Varner
//
//  This file is part of Vamos Automotive Simulator.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _PARTICLE_H_
#define _PARTICLE_H_

#include <vamos/geometry/Three_Vector.h>
#include <vamos/geometry/Three_Matrix.h>
#include <vamos/geometry/Inertia_Tensor.h>
#include <vamos/geometry/Surface.h>
#include <vamos/body/Component.h>
#include <vamos/body/Frame.h>

namespace Vamos_Body
{
  //* A Particle is a massive particle which is part of a rigid body.
  // It has position and orientation information inherited from Frame.
  // The parent frame is the rigid body's.  A Particle can exert
  // forces and torques on the rigid body because it is derived from
  // Component.
  class Particle : public Component, public Frame
  {
  protected:
	// The mass of the particle.
	double m_mass;

	// The current impulse (force integrated over time).  m_impulse is
	// typically divided by the time step and applied for a single
	// time step.
	Vamos_Geometry::Three_Vector m_impulse;

	// The static coefficient of friction.
	double m_static_friction;

	// The sliding coefficient of friction.
	double m_sliding_friction;

	// The coefficient of restitution.
	double m_restitution;

	// true if a collision has occurred.
	bool m_contact;

  public:
	//** Constructors

	// Specify position and orientation.
	Particle (double mass, const Vamos_Geometry::Three_Vector& position, 
			  const Vamos_Geometry::Three_Matrix& orientation, 
			  double stat, double sliding, double restitution);

	// Take the parent's orientation.
	Particle (double mass, const Vamos_Geometry::Three_Vector& position, 
			  double stat, double sliding, double restitution);

	// The particle's frame is coincident with the parent's.
	Particle (double mass);


	//** Destructor
	virtual ~Particle () {};

	// Propagate the Component forward in time by TIME and update the
	// force and torque.
	virtual void propagate (double time);

	// Return the force exerted on the rigid body in the body's frame.
	virtual Vamos_Geometry::Three_Vector force () const
	{ return rotate_out (m_force); }

	// Return the torque exerted on the rigid body in the body's frame.
	virtual Vamos_Geometry::Three_Vector torque () const
	{ return rotate_out (m_torque); }

	// Classes derived from Particle may lie about their positions
	// for collisions...
	virtual Vamos_Geometry::Three_Vector contact_position () const 
	{ return position (); }

	// ...for exerting forces...
	virtual Vamos_Geometry::Three_Vector force_position () const 
	{ return position (); }

	// ...or for constructing the inertia tensor of the rigid body.
	virtual Vamos_Geometry::Three_Vector mass_position () const 
	{ return position (); }
  
	// Set the force, impulse and torque to zero;
	virtual void reset ();

	// Handle collisions.  The return value is how much the particle
	// has moved in the z-direction as a result of the contact.  For a
	// Particle, this is always 0.  But, for derived classes that
	// model moving parts, a non-zero value may be returned.
	virtual Vamos_Geometry::
	Three_Vector contact (const Vamos_Geometry::Three_Vector& position,
						  const Vamos_Geometry::Inertia_Tensor& inertia,
						  const Vamos_Geometry::Three_Vector& velocity,
						  const Vamos_Geometry::Three_Vector& distance,
						  double ang_velocity,
						  const Vamos_Geometry::Surface&);

	// Return the particle's mass.
	double mass () const { return m_mass; }
  };
}

#endif // !_PARTICLE_H_
