//  Tire.h - the tire for a wheel.
//
//  Copyright (C) 2001 Sam Varner
//
//  This file is part of Vamos Automotive Simulator.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _TIRE_H_
#define _TIRE_H_

#include <vamos/geometry/Surface.h>
#include <vamos/geometry/Three_Vector.h>
#include <vamos/body/Component.h>
#include <plib/sl.h>

using Vamos_Geometry::Three_Vector;

namespace Vamos_Body
{
  //* The tire for a wheel.
  class Tire : public Component
  {
	// The radius of the tire.
	double m_radius;

	// Rolling resistance on a hard surface.
	double m_rolling_resistance;

	// The maximum coefficient of friction on a hard surface.
	double m_friction;

	//
	double m_forgiveness;

	// The rotational inertia of the tire.
	double m_inertia;

	// The torque that the tire exerts on the wheel due to friction.
	double m_torque;

	// The rotational speed of the tire in radians per second.
	double m_rotational_speed;

	// How fast the tire is sliding.
	double m_slide;

	//// Input paremeters provided by the wheel.

	// The velocity of the wheel relative to the road.
	Three_Vector m_velocity;

	// The angular velocity about the normal to the surface.
	double m_normal_ang_velocity;

	// The force perpendicular to the surface.
	double m_normal_force;

	// The current camber, supplied by the wheel.
	double m_camber;

	// True if the brake for this wheel is locked.
	bool m_is_locked;

	// The surface that the tire is currently on.
	const Vamos_Geometry::Surface* mp_surface;

  public:
	//** Constructor
	Tire (double radius, double rolling, double friction, double forgiveness, 
		  double inertia);

	//** Destructor
	virtual ~Tire ();

	// Called by the wheel to update the tire.
	void input (const Three_Vector& velocity, 
				double normal_ang_velocity,
				double normal_force,
				double camber,
				bool is_locked,
				const Vamos_Geometry::Surface& surf);

	// Called by the wheel to give the torque on the wheel due do
	// acceleration or braking.
	void torque (double torque) { m_torque = torque; }

	// Advance the tire in time by TIME.
	void propagate (double time);

	// Return the current torque due to friction.
	double torque () { return m_torque; }

	// Return the radius of the tire
	double radius () const { return m_radius; }

	// Return the rotational speed of the tire.
	double rotational_speed () const { return m_rotational_speed; }

	// Return the linear speed of the tread.
	double speed () const { return m_rotational_speed * m_radius; }

	// Return the linear sliding speed. 
	double slide () const { return m_slide; }

	// Return the position of the contact patch in the wheel's
	// coordinate system.
	Three_Vector contact_position () const;

	// Set the tire to its initial conditions.
	void reset ();
  };
}

#endif // !_TIRE_H_
