//  Wheel.h - a wheel.
//
//  Copyright (C) 2001--2002 Sam Varner
//
//  This file is part of Vamos Automotive Simulator.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _WHEEL_H_
#define _WHEEL_H_

#include <vamos/body/Suspension.h>
#include <vamos/body/Tire.h>
#include <vamos/body/Brake.h>
#include <vamos/body/Particle.h>
#include <vamos/geometry/Surface.h>
#include <vamos/geometry/Three_Vector.h>
#include <vamos/geometry/Inertia_Tensor.h>

using Vamos_Geometry::Three_Vector;

namespace Vamos_Body
{
  //* A wheel.  
  // Handles wheel geometry.  See Tire, Brake and Suspension for
  // related functionality.
  class Wheel : public Particle
  {
  public:
	//** The Side enumeration must be defined before the m_side
	//member is declared.  More public declarations follow the private
	//section.  

	// Enumeration values for specifying which side of the car the
	// weel is on.
	enum Side { LEFT, RIGHT };

  private:
	//** Constants axes for changes in orientation.

	// The axis of rotation for camber adjustments.
	const static Three_Vector CAMBER_AXIS;

	// The axis of rotation for caster adjustments.
	const static Three_Vector CASTER_AXIS;

	// The axis of rotation for steering.
	const static Three_Vector STEER_AXIS;


	// The initial position of the wheel.
	Three_Vector m_original_position;

	// The side, RIGHT or LEFT, that the tire is on.  This is used to
	// choose the right direction for camber, caster and toe
	// adjustments. 
	Side m_side;

	// The suspension that the wheel is attached to.
	Suspension m_suspension;

	// The tire that is attached to the wheel.
	Tire m_tire;

	// The brake for the wheel.
	Brake m_brake;

	// The current rotational velocity of the wheel.
	double m_rotational_velocity;

	// The velocity of the ground relative to the wheel.
	Three_Vector m_ground_velocity;

	// The current normal vector for the road.
	Three_Vector m_normal;

	// The angular velocity about m_normal.
	double m_normal_ang_velocity;
	
	// The surface we're currently on
	Vamos_Geometry::Surface m_surface;

	// The orientation of the wheel in the absence of steering and
	// displacement.
	Three_Matrix m_static_orientation;

	// The static camber angle in radians.  Positive camber is a
	// rotation that moves the tops of the tires away from the center
	// of the car.  So the direction of the rotation depends on the
	// value of m_side.
	double m_camber;

	// The current camber angle in radians.
	double m_current_camber;

	// The caster angle in radians.
	double m_caster;

	// The toe angle in radians.  Positive is toe-in.
	double m_toe;

	// The deflection of the tire in radians due to steering.
	double m_steer_angle;
	
  public:
	//** Constructor
	Wheel (double mass, 
		   Three_Vector position, 
		   Side side_of_car, 
		   double restitution,
		   const Suspension& suspension, 
		   const Tire& tire, 
		   const Brake& brake);

	// Advance the wheel forward in time by TIME.
	void propagate (double time);

	// Handle hard collisions.  The return value is the displacement
	// of the particle as a result of the contact.  This function is
	// called by a Body object.
	Three_Vector contact (const Three_Vector& position,
						  const Vamos_Geometry::Inertia_Tensor& inertia,
						  const Three_Vector& vecocity, 
						  const Three_Vector& distance,
						  double ang_velocity,
						  const Vamos_Geometry::Surface& surface);

	// Handle soft collisions.  The return value is the displacement
	// of the particle as a result of the contact.  This function is
	// called by a Car object.
	Three_Vector wheel_contact (const Three_Vector& position,
								const Vamos_Geometry::Inertia_Tensor& inertia,
								const Three_Vector& velocity, 
								const Three_Vector& distance,
								double ang_velocity,
								const Vamos_Geometry::Surface& surface);

	// Connect an anti-roll bar with another wheel.
	void anti_roll (Wheel* other, double spring_constant);

	// Return the torque exerted on wheel as a result of friction
	// between the tire and the surface.
	Three_Vector torque () 
	{ return Three_Vector (0.0, m_tire.torque (), 0.0); }

	// Apply the torque TORQUE_IN to the tire.  This torque results
	// from acceleration or braking.
	void torque (double torque_in);

	// Return the torque exerted on the wheel by the brake when a
	// pressure of FACTOR * Brake::m_max_pressure is applied to the
	// brake.
	double brake_torque (double factor);

	// Apply the brake with a pressure of FACTOR *
	// Brake::m_max_pressure.
	void brake (double factor);

	//** Geometry-setting methods.  The arguments are in degrees.

	// Set the steering angle.
	void steer (double degree_angle);

	// Set the camber angle.
	void camber (double degree_angle);

	// Set the caster angle.
	void caster (double degree_angle);

	// Set the toe angle.
	void toe (double degree_angle);


	// Return the speed of the contact patch of the tire with respect
	// to the ground.
	double slide () const { return m_tire.slide (); }

	// Return the wheel's rotational speed in radians per second.
	double rotational_speed () const { return m_tire.rotational_speed (); }

	// Return the linear speed of the tire tread.  This is the speed
	// that a speedometer on this wheel would read.
	double speed () const { return m_tire.speed (); }

	// Return the position relative to the body where the wheel exerts
	// its force.
	Three_Vector force_position () const;

	// Return the position of the wheel relative to the body for the
	// purpose of detecting collisions.
	Three_Vector contact_position () const;

	// Return the wheel to its initial conditions.
	void reset ();
  };
}

#endif // !_WHEEL_H_
